// 2007/09/15 Hiroyuki Ogasawara
// vim:ts=4 sw=4:
/*!
	@file
	@date	2007/09/15
	@author	Hiroyuki Ogasawara
	@brief

	Shader  Hello World!

	@sa http://dench.flatlib.jp/
	@sa http://wlog.flatlib.jp/
*/


#include	<windows.h>
#include	<d3d10.h>
#include	<d3dx10.h>

#define		ZRELEASE( inf )		((inf)?((inf)->Release(),(inf)=NULL):0)


#define		_APP_TITLE_		TEXT("Wheel Handle ss02")

enum {
	WINDOW_SIZE_WIDTH	= 1024,
	WINDOW_SIZE_HEIGHT	= 768,
};



static IDXGISwapChain*			g_iSwapChain= NULL;
static ID3D10Device*			g_iDevice= NULL;
static ID3D10RenderTargetView*	g_iRenderTargetView= NULL;
static ID3D10DepthStencilView*	g_iDepthStencilView= NULL;
static ID3D10Effect*			g_iEffect= NULL;
static ID3D10InputLayout*		g_iInputLayout= NULL;
static ID3D10Buffer*			g_iStringBuffer= NULL;
static ID3D10ShaderResourceView*	g_iShaderResourceView= NULL;

static float	g_Rotate= 0.0f;


/*!
	@param[in]	hwnd
	@param[in]	winmode
	@param[in]	width
	@param[in]	height
	@brief
		Direct3D10 ̎蔲

	R[hZ邱ƍŗDŃG[`FbN܂B
*/
static void
InitDevice( HWND hwnd, int winmode, int width, int height )
{
	// Device ̍쐬
	DXGI_SWAP_CHAIN_DESC	swapdesc= {
		{
			width, height,
			{	60,	1,	},
			DXGI_FORMAT_R8G8B8A8_UNORM,
			DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED,
			DXGI_MODE_SCALING_UNSPECIFIED,
		},
		{	1,	0,	},	// sample
		DXGI_USAGE_RENDER_TARGET_OUTPUT,
		1,
		hwnd,
		winmode,
		DXGI_SWAP_EFFECT_DISCARD,
		0
	};
	D3D10CreateDeviceAndSwapChain(
			NULL,
			D3D10_DRIVER_TYPE_HARDWARE,
			NULL,
			D3D10_CREATE_DEVICE_DEBUG,
			D3D10_SDK_VERSION,
			&swapdesc,
			&g_iSwapChain,
			&g_iDevice
		);


	// FrameBuffer ̍쐬
	ID3D10Texture2D*	iBackBuffer= NULL;
	g_iSwapChain->GetBuffer( 0, __uuidof( ID3D10Texture2D ),
							reinterpret_cast<void**>( &iBackBuffer ) );
	g_iDevice->CreateRenderTargetView( iBackBuffer, NULL, &g_iRenderTargetView );
	ZRELEASE( iBackBuffer );

	ID3D10Texture2D*	iTexture= NULL;
	D3D10_TEXTURE2D_DESC	tex2ddesc= {
		width, height, 1, 1,	// mip, array
		DXGI_FORMAT_D32_FLOAT,
		{	1,	0,	},	// sample
		D3D10_USAGE_DEFAULT,
		D3D10_BIND_DEPTH_STENCIL,
		0,	// cpu
		0	// misc
	};
	g_iDevice->CreateTexture2D( &tex2ddesc, NULL, &iTexture );

	D3D10_DEPTH_STENCIL_VIEW_DESC	vdesc= {
		DXGI_FORMAT_D32_FLOAT,
		D3D10_DSV_DIMENSION_TEXTURE2D
	};
	g_iDevice->CreateDepthStencilView( iTexture, &vdesc, &g_iDepthStencilView );
	ZRELEASE( iTexture );
	g_iDevice->OMSetRenderTargets( 1,
						&g_iRenderTargetView, g_iDepthStencilView );


	// Viewport ̐ݒ
	D3D10_VIEWPORT	viewport= {
		0, 0, width, height, 0.0f, 1.0f
	};
	g_iDevice->RSSetViewports( 1, &viewport );


	// Shader ̓ǂݍ
	ID3D10Blob*	iblob= NULL;
	if( FAILED( D3DX10CreateEffectFromFile(
				TEXT("font.fx"),
				NULL, NULL, "fx_4_0", 0, 0, g_iDevice,
				NULL, NULL, &g_iEffect, &iblob, NULL ) ) ){
		OutputDebugStringA( reinterpret_cast<char*>(
							iblob->GetBufferPointer() ) );
		ZRELEASE( iblob );
	}


	// Shader ̏lo^
	D3DXMATRIX	projection;
	D3DXMatrixPerspectiveFovLH( &projection,
			3.141592f/3.0f,
			WINDOW_SIZE_WIDTH/(float)WINDOW_SIZE_HEIGHT,
			1.0f, 10000.0f );
	g_iEffect->GetVariableByName( "WorldToProjection" )->AsMatrix()
					->SetMatrix( (float*)&projection );


	// InputLayout ̍쐬
	static const D3D10_INPUT_ELEMENT_DESC	_inputDesc[]= {
		{	"CODE", 0,	DXGI_FORMAT_R8_UINT, 0, 0,
									D3D10_INPUT_PER_VERTEX_DATA, 0,	},
	};
	D3D10_PASS_DESC	pdesc;
	g_iEffect->GetTechniqueByIndex(0)->GetPassByIndex(0)->GetDesc( &pdesc );
	g_iDevice->CreateInputLayout( _inputDesc, 1,
			pdesc.pIAInputSignature, pdesc.IAInputSignatureSize,
			&g_iInputLayout );


	// p Buffer 쐬
	D3D10_BUFFER_DESC	bdesc= {
		256,
		D3D10_USAGE_DYNAMIC,
		D3D10_BIND_VERTEX_BUFFER,
		D3D10_CPU_ACCESS_WRITE,
		0,
	};
	g_iDevice->CreateBuffer( &bdesc, NULL, &g_iStringBuffer );


	// Font f[^̓ǂݍ
	D3DX10CreateShaderResourceViewFromFile( g_iDevice,
			TEXT("fontdata.png"),
			NULL, NULL, &g_iShaderResourceView, NULL );
	g_iEffect->GetVariableByName( "FontTexture" )->AsShaderResource()->
			SetResource( g_iShaderResourceView );

}


static void
ReleaseDevice()
{
	ZRELEASE( g_iShaderResourceView );
	ZRELEASE( g_iInputLayout );
	ZRELEASE( g_iRenderTargetView );
	ZRELEASE( g_iDepthStencilView );
	ZRELEASE( g_iDevice );
	ZRELEASE( g_iSwapChain );
}


/*!
	@brief
		`惋[v
*/
static void
Render()
{
	// FrameBuffer NA
	static const float	color[4]= {
		0.0f, 0.0f, 1.0f, 0.0f
	};
	g_iDevice->ClearRenderTargetView( g_iRenderTargetView, color );
	g_iDevice->ClearDepthStencilView( g_iDepthStencilView,
						D3D10_CLEAR_DEPTH|D3D10_CLEAR_STENCIL, 1.0f, 0 );


	// ̕`ʒuݒ
	float	loc[4]= {
		-5.0f, 0.0f, 20.0f + sinf( g_Rotate )*14, 0.0f
	};
	g_iEffect->GetVariableByName( "Locate" )->AsVector()->SetFloatVector( loc );
	g_Rotate+= 3.141592f*0.03f;

	// VF[_[Kp
	g_iEffect->GetTechniqueByIndex( 0 )->GetPassByIndex( 0 )->Apply( 0 );


	// `敶̏
	const char*	Message= "HELLO WORLD!";

	void*	ptr;
	g_iStringBuffer->Map( D3D10_MAP_WRITE_DISCARD, 0, &ptr );
	strcpy( reinterpret_cast<char*>( ptr ), Message );	// !!
	g_iStringBuffer->Unmap();


	// `p͒lݒ
	g_iDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_POINTLIST );
	g_iDevice->IASetInputLayout( g_iInputLayout );
	UINT	vSize= sizeof(char);	// !!
	UINT	vOffset= 0;
	g_iDevice->IASetVertexBuffers( 0, 1, &g_iStringBuffer, &vSize, &vOffset );


	// ̕`
	g_iDevice->Draw( strlen( Message ), 0 );	// !!


	g_iSwapChain->Present( 0, 0 );
}



LRESULT CALLBACK
WindowExec( HWND hwnd, UINT mes, WPARAM wparam, LPARAM lparam )
{
	switch( mes ){
	case WM_DESTROY:
		PostQuitMessage( 0 );
		return	FALSE;
	}
	return	DefWindowProc( hwnd, mes, wparam, lparam );
}


/*!
	@brief
		蔲main
*/
int APIENTRY
WinMain( HINSTANCE hinstance, HINSTANCE hpinstance, LPSTR cmdline, int show )
{

	WNDCLASS	wc;
	wc.style= CS_HREDRAW|CS_VREDRAW;
	wc.lpfnWndProc= WindowExec;
	wc.cbClsExtra= 0;
	wc.cbWndExtra= 0;
	wc.hInstance= hinstance;
	wc.hIcon= NULL;
	wc.hCursor= LoadCursor( NULL, IDC_ARROW );
	wc.hbrBackground= (HBRUSH)GetStockObject( BLACK_BRUSH );
	wc.lpszMenuName= NULL;
	wc.lpszClassName= _APP_TITLE_;
	RegisterClass( &wc );

	HWND	hWnd= CreateWindowEx( WS_EX_APPWINDOW,
					_APP_TITLE_, _APP_TITLE_,
					WS_OVERLAPPEDWINDOW,
					CW_USEDEFAULT, CW_USEDEFAULT,
					WINDOW_SIZE_WIDTH, WINDOW_SIZE_HEIGHT,
					NULL, NULL, hinstance, NULL );
	ShowWindow( hWnd, SW_SHOWNORMAL );
	UpdateWindow( hWnd );
	InitDevice( hWnd, TRUE, WINDOW_SIZE_WIDTH, WINDOW_SIZE_HEIGHT );

	for(;;){
		for( MSG msg ; PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) ;){
			if( msg.message == WM_QUIT ){
				ReleaseDevice();
				return	0;
			}
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
		Render();
		Sleep( 16 );
	}

	return	0;
}

